% BOOTSTRAPPER Parameteric bootstrap for the exponential search model. 
% Parameters p0, pn, betap, gammap, and dinf are re-estimated from simulated
% data.
% INPUTS:
% parameters is a vector of model parameters containing 
% [p0 alpha K pn betap gammap dinf]
% Ainit is the initial two-mode adjacency matrix,
% nbs is the number of simulation runs/spells in the bootstrap 
% (number of times paramters are refitted to synthetic data),
% ntime is the number of periods in each simulation
% nsims is the number of simulations used in the optimization of paramters
% rstate is the state of the random number generator
% OUTPUTS:
% standard errors of the parameters are named as seX for variable X,
% estimated parameters of different runs stored in vectors Xs of length nbs
% REQUIREMENTS: add2dist, fastsetdiff, findallneighbors, newrandpeople, 
% openvacancies, optimizer, simuloop, updateadj
function [sep0,sepn,sebetap,segammap,sedinf,pns,betaps,gammaps,dinfs]=bootstrapper(parameters,pout,Ainit,nbs,ntime,nsims,cmax,tolx,lbounds,ubounds,rstate)

%Set this flag to true if you want to use octave standard optimization routine
sflag==false;

K=parameters(1);
pn=parameters(2);
betap=parameters(3);
gammap=parameters(4);
dinf=parameters(5);

if nargin<=8 || isempty(lbounds)
lbounds=[];
ubounds=[];
end

if nargin<11 || isempty(rstate)
rstateinit=rand('state');
else
rstateinit=rstate;
end

%Bootstrapping
%Creating artificial data, this is parallelized
A=Ainit;
[distd,degnewd]=pararrayfun(nproc,@(seed) simuloop(Ainit,pn,pout,betap,gammap,dinf,K,ntime,cmax,rstate,seed),[4*nsims:1:4*nsims+nbs],"Uniformoutput",false,"VerboseLevel",0); %Note that the seeding is different from the one used in optimizer; make sure that nsim in the bootstrapper is at least 1/4 of nsim used in the initial estimation of the parameters to avoid overlap of seed values

%Getting the distributions and pn and p0 and refitting the model
m=sum(sum(Ainit));
for j=1:nbs 
dis=distd{j}(2:end)./sum(distd{j}(2:end));
deg=degnewd{j}(2:end)./sum(degnewd{j}(2:end));
pns(j)=distd{j}(1)/sum(distd{j});
p0s(j)=full(sum(degnewd{j})/(ntime*m));
[betaps(j),gammaps(j),dinfs(j)]=optimizer(betap,gammap,dinf,pns(j),pout,K,dis,deg,Ainit,nsims,ntime,cmax,tolx,lbounds,ubounds,rstate);
if sflag==true
f = @(params)simfitObj(params,pns(j),pout,K,dis,deg,Ainit,nsims,ntime,cmax,rstate);
options2 = optimset('TolX', 0.001);
[xopt,optval]=fminunc(f,[betaps(j) gammaps(j) dinfs(j)]',options2);
betaps(j)=xopt(1);
gammaps(j)=xopt(2);
dinfs(j)=xopt(3);
end
%Note: this is not parallelized here because simuloop inside optimizer/simobj is parallelized
end

%Standard deviations
sepn=std(pns);
sebetap=std(betaps);
segammap=std(gammaps);
sedinf=std(dinfs);
