% GETDESCRIPTIVESTATS extracts descriptive statistics from a period between 
% fyr and lyr given the set of firms listed in fimlist.
% INPUTS: 
% fyr first year, 
% lyr last year, 
% firmlist is the list of firms in the sample, 
% caseflag and eflag can be left empty (see below) when firmlist is given as
% an input argument. eflag is related to requiring the existence of firms 
% throughout the sample period and caseflag is related to the selection of 
% firms (see the help of firmselection for details).
% The edgelists of different years should be stored in files of the form 
% edgelistyr.tsv (yr for year)
% OUTPUTS: 
% output is a cell array with the following contents 
% descriptives{1,yercount}=degree histogram of people 
% (degree=number of board positions)
% descriptives{2,yercount}=distribution of distances of new individuals, 
% note THAT the last value is the number of persons not found in any neighborhood,
% the last element contains both individuals that are complete outsiders and 
% those who are from a different component ,
% descriptives{3,yercount}=degree distribution of old individuals
% descriptives{4,yercount}=degree distribution of new individuals conditional on 
% that they are found in some neighborhood,
% descriptives{5,yercount}=number of occupied positions in each firm in each year,
% descriptives{6,yercount}=ndeg; is the degree distribution of all positions 
% (including empty ones that are treated as degree one positions),
% descriptives{7,yercount}=ndegold; is the degree distribution of opened 
% vacancies (including opened empty ones),
% descriptives{8,yearcount}=yearly average clustering coefficient for 
% the firm projection,
% descriptives{9,yearcount}=empty positions for each firm 
% (relative to max positions) in each year, see line 147
% descriptives{10,1}=[totalnew,totalopens], how many new people there are and 
% how many vacancies have been opened (during the whole time period, here 
% totalopens does not include empty seats that were filled, but includes 
% seats left empty) 
% descriptives{11,1}=maximum number of positions
% descriptives{12,1} is the seat distribution
% Example of a call: descriptives=getdescriptivestatsPC(4000,4001);
% Note 1: for elements 1, 5, 6, 8, 9 yearcount goes one step further 
% than for others
% Note 2: there is a difference when referring to individuals and positions; 
% a person may have left multiple positions simultaneously, each opened vacancy
% is treated separately. Hence, there are less people who have left compared to 
% opened vacancies.
% REQUIREMENTS: add2dist, appendadj, clustCoeffAlt, elist2adj, fastsetdiff, 
% findinneighborhoodvac, firmsinpc, getdegrs, getdegrsSimple, gianComponentAlt, 
% unionidvs + functions required in giantComponentAlt
function descriptives=getdescriptivestats(fyr,lyr,firmlist,caseflag,eflag)

if nargin<=4 || isempty(eflag)
eflag=false;
end

if nargin<3 || isempty(firmlist)
if nargin<4 || isempty(caseflag)
caseflag=[];
end
if nargin<5 || isempty(eflag)
caseflag=[];
end
firmlist=firmsinpc(fyr,lyr,caseflag,eflag);%on caseflag and eflag see firmsinpc.m
end

%Some initializations
totalnew=0;
totalopens=0;
seats=zeros(length(firmlist),lyr-fyr+1);
seatdist=[];

%Loop through years from fyr to lyr
for yr=fyr:1:lyr-1

if yr==fyr; %treating the first year separately
[firms1,individuals1,A1]=elist2adj(yr);%make adjacency matrices from edgelists. Warning: this takes some time.
if eflag==false %this flag controls the "existence" assumption, if true then it does not matter whether a firms has quit
[firms1rest,indA,indB]=intersect(firms1,firmlist);
A1=A1(indA,:);
else
A1=appendadj(A1,firms1,firmlist);%this function appends all the firms in firmlist into rows of A1
end %end of if eflag==false

indic=any(A1);
individuals1=individuals1(indic);
A1(:,!indic)=[]; %Some of the individuals have no positions in the forms listed in firmlist

descriptives{5,yr-fyr+1}=sum(A1,2);
descriptives{8,yr-fyr+1}=clustCoeffAlt(spones(A1*A1')-diag(ones(size(A1,1),1)));
seats(:,yr-fyr+1)=sum(A1,2);
seatdist=add2dist(seats(:,yr-fyr+1),seatdist,0);
else
A1=A1temp;
individuals1=individuals1temp;
end%end of if yr==fyr

[firms2,individuals2,A2]=elist2adj(yr+1); %find the next adjacency matrix of next year
if eflag==false
[firms2rest,indA,indB]=intersect(firms2,firmlist);
A2=A2(indA,:);
else
A2=appendadj(A2,firms2,firmlist);%this function appends all the firms in firmlist into rows of A2
end

indic=any(A2);
individuals2=individuals2(indic);
A2(:,!indic)=[];
descriptives{5,yr-fyr+2}=sum(A2,2);
descriptives{8,yr-fyr+2}=clustCoeffAlt(spones(A2*A2')-diag(ones(size(A2,1),1)));
seats(:,yr-fyr+2)=sum(A2,2);
seatdist=add2dist(seats(:,yr-fyr+2)',seatdist,0);
A1temp=A2; %save the next year

individuals1temp=individuals2; 

%get the descriptives over different years
degreef=sum(A1,1);
descriptives{1,yr-fyr+1}=histc(degreef,[1:1:max(degreef)]);
[A1,A2,indoldidvs,indnewidvs]=unionidvs(individuals1,individuals2,A1,A2); %take all the people in both years
templmat=(A1==1 & A2==0);%people who have lost a position
compchange=find(sum(templmat,2)~=0);%companies for which board members have changed

oldidvs=find(sum(templmat,1)~=0);%individuals who have moved out from at least one board
[ndeg,ndegold,nnew]=getdegrs(A1,A2);
descriptives{6,yr-fyr+1}=ndeg;
descriptives{7,yr-fyr+1}=ndegold;%this statistics is very important; it can be used to obtain the number of vacancies that were opened

templmat=(A1==0 & A2==1); %this indicator matrix contains individuals who did not have any position in A1 but who have one in A2
newidvs=find(sum(templmat,1)~=0);
[descriptives{2,yr-fyr+1},descriptives{4,yr-fyr+1}]=findinneighborhoodvac(compchange,templmat,A1);
descriptives{2,yr-fyr+1}(end)=descriptives{2,yr-fyr+1}(end)-(sum(descriptives{2,yr-fyr+1})-sum(descriptives{4,yr-fyr+1})); %the last element of the distance histogram is set to the number of people who were not found in any neighborhood but who had a position in the previous period
%remember that if a person is not found he/she can be from a different component than the firm with the vacancy
tempdmat=sum(A1(:,oldidvs));
descriptives{3,yr-fyr+1}=histc(tempdmat,[1:1:max(tempdmat)]);

if yr==lyr-1
degreef=sum(A2,1);
descriptives{1,lyr-fyr+1}=histc(degreef,[1:1:max(degreef)]);
ndeg=getdegrsSimple(A2);
descriptives{6,lyr-fyr+1}=ndeg;
end

%change in the number of positions
totalnew=totalnew+nnew; 
totalopens=totalopens+sum(ndegold);
end % end of for
descriptives{10,1}=[totalnew,totalopens];

maxseats=max(seats,[],2);
descriptives{11,1}=maxseats;
for yr=fyr:lyr
descriptives{9,yr-fyr+1}=maxseats-seats(:,yr-fyr+1);
%the next lines are to correct for filling of empty positions. 
%Empty positions are treated as "degree one" positions as if there is someone who has been kicked out when filling an empty seat.
if yr<lyr
%Find firms who have filled some empty vacancies
seatdif=seats(:,yr-fyr+2)-seats(:,yr-fyr+1);
indic=(seatdif<0);%some seats are left empty
descriptives{4,yr-fyr+1}(1)=descriptives{4,yr-fyr+1}(1)-sum(seatdif(indic));
end
%Add the number of empty seats to the total number of positions with degree one, add these also to the degree distribution of people, as if there is one individual holding each position.
ndeg=descriptives{6,yr-fyr+1};
ndeg(1)=ndeg(1)+sum(descriptives{9,yr-fyr+1});
descriptives{6,yr-fyr+1}=ndeg;
descriptives{1,yr-fyr+1}(1)=descriptives{1,yr-fyr+1}(1)+sum(descriptives{9,yr-fyr+1});
end
descriptives{12,1}=seatdist;
