% FITSEARCHMODEL Main caller for fitting the exponential search model into data. 
% The main elements needed in the fitting are the empirical distributions of 
% distances and degrees of newly recruited board members. This data can be 
% obtained from a set of edgelists by using getddescriptivestats and 
% aggregatedist. If the arguments firmlist, descriptives, aggdescriptives are 
% left empty these are obtained by using the aforementioned functions. 
% The edgelists are are assumed to be stored in format edgelistyr.tsv, 
% for further details see the help of descriptivestats.
% INPUTS: 
% fyr=first year, lyr=last year
% firmlist constains the list of firms in the main component 
% (can be left empty if not known)
% descriptives contains the descriptive statistics of the data 
% (can be left empty, see getdescriptivestats.m)
% aggdescriptives contains aggregated data of descriptive statistics 
% (can be left empty, see aggregatedist.m)
% Ainit is the adjacency matrix corresponding to year fyr 
% (can be left empty if not known, see initmatgc.m)
% fitoptions is a structure that contains the options used in fitting the 
% parameters, following fields are available:
% initial values of the parameters initalpha, initbeta, and initgamma, initdinf
% saveflag; set true if data is to be saved)
% plotflag; set true if figures are to be drawn
% bsflag; set true if bootstrapped standard errors are needed, 
% warning: bootstrapping may take days or even weeks, 
% it is recommended to run the bootstraps separately
% rmflag; set true if 'random model', i.e., model where distance and degree 
% do not matter, is to be simulated 
% nsims; number of simulations for one single evaluation of the objective 
% function.
% nbs; number of simulations in the bootstrap
% caseflag: the selection of the sample, caseflag=0 takes the firms in 
% the main component throughout the sample period, caseflag=1 takes the union 
% of firms in the main component at some point, caseflag=2 takes all the firms
% cmax; maximum number of neighborhoods that is went through
% initflag; set true if you want to use the results of optimizer as initial 
% values for fminunc
% OUTPUTS:
% parameters is a vector containing the estimated parameters; 
% parameters=[p0 alpha K pn betap gammap dinf], where
% p0 probability of opening a vacancy (also influenced by parameter alpha), 
% pn probability of recruiting outsiders,
% K is the maximal degree, 
% betap, gammap and dinfty are the parameters of the model,
% sers contains standars errors if they are bootstrapped
% diagnostics is a cell array containing distance and degree distributions 
% of new individuals
% r2 contains r-squareds, r2(1) is r2 for explaining the degree distribution 
% of new individuals and r2(2) for explaining the distance distribution of new individuals
% REQUIREMENTS: parallel toolbox and the whole package of m-files
function [parameters, sers, diagnostics, r2]=fitsearchmodel(fyr,lyr,firmlist,descriptives,aggdescriptives,Ainit,fitoptions)

if nargin<7
fitoptions=[]; %use default options
end

[initalpha,initbeta,initgamma,initdinf,nsims,bsflag,nbs,cmax,caseflag,alphaflag,saveflag,filename,rmflag,plotflag,initflag,tolx]=parsefitoptions(fitoptions);

if nargin<=2 || isempty(firmlist)
firmlist=firmselection(fyr,lyr,[],caseflag);
descriptives=getdescriptivestats(fyr,lyr,firmlist);
end

if nargin<=4 || isempty(aggdescriptives)
aggdescriptives=aggregatedist(descriptives,true);
end

refdist=aggdescriptives{10};
if alphaflag==true
f = @(params)alphafitObj(params,refdist);
[xopt,optval]=fminunc(f,[aggdescriptives{11} initalpha]');
end

if isempty(Ainit) || nargin<=5
Ainit=initmatgc(fyr,descriptives{11,1},firmlist);
end

ddist=aggdescriptives{1};
ccoef=aggdescriptives{8};
didisnew=aggdescriptives{2};
dedisnew=aggdescriptives{4};

pout=aggdescriptives{10};

ntime=lyr-fyr; %number of periods to be simulated
K=length(aggdescriptives{1});%the maximum degree
pn=aggdescriptives{9}; %probabiity of recruiting outsiders
diagnostics=[];
%K=10;
rstate=rand('state');%stroring a random initial state
[betap,gammap,dinf,diagnostics]=optimizer(initbeta,initgamma,initdinf,pn,pout,K,didisnew,dedisnew,Ainit,nsims,ntime,cmax,tolx,rstate);

%Using the outcomes of optimizer as initial values for fminunc
if initflag==true
f = @(params)simfitObj(params,pn,pout,K,didisnew,dedisnew,Ainit,nsims,ntime,cmax,rstate);
options2 = optimset('TolX', 0.001);
[xopt,optval]=fminunc(f,[betap gammap dinf]',options2);%or alternatively
%[xopt,optval]=fminsearch(f,['betap gammap dinf]',options2);
betap=xopt(1);
gammap=xopt(2);
dinf=xopt(3);
end

%Output parameters:
parameters=full([p0 alpha K pn betap gammap dinf]);

%r-squareds
len1=length(diagnostics{2});
len2=length(aggdescriptives{4});

if len1>len2
tempvec=[aggdescriptives{4} zeros(1,len1-len2)];
avrdist=mean([aggdescriptives{4} zeros(1,len1-len2)]);
errs1=norm(diagnostics{2}-tempvec)^2;
errs2=norm(tempvec-avrdist)^2;
elseif len2>len1
tempvec=[diagnostics{2} zeros(1,len2-len1)];
avrdist=mean(aggdescriptives{4});
errs1=norm(tempvec-aggdescriptives{4})^2;
errs2=norm(aggdescriptives{4}-avrdist)^2;
elseif len2==len1
avrdist=mean(aggdescriptives{4});
errs1=norm(diagnostics{2}-aggdescriptives{4})^2;
errs2=norm(aggdescriptives{4}-avrdist)^2;
end
r2(1)=1-errs1/errs2; %this is r2 for explaining the degree distribution of new individuals
%and this is r2 for explaining the distance distribution of new individuals
r2(2)=1-norm(diagnostics{1}-aggdescriptives{2})^2/norm(aggdescriptives{2}-mean(aggdescriptives{2}))^2;

%Simulating the 'random' model for comparison
if rmflag==true
[Ar,Afout,dis0new,ravrccoef,diagrmodel]=simulator(Ainit,pn,pout,[],[],dinf,dione,mflag,K,lyr-fyr,[],[],[],nsims,cmax,rstate);
else
diagrmodel=[];
end

%Bootstrapping the model
if bsflag==true
%Warning bootstrapping really takes time
[sep0,sepn,sebetap,segammap,sedinf,pns,p0s,betaps,gammaps,dinfs]=bootstrapper(parameters,pout,Ainit,nbs,ntime,nsims,cmax,tolx,rstate)
sers(1)=sep0;
sers(2)=sepn;
sers(3)=sebetap;
sers(4)=segammap;
sers(5)=sedinf;
else
sers=[];
end

%Saving the outputs
if saveflag==true
save('-binary',outfilename,'firmlist','descriptives','aggdescriptives','parameters','sers','diagnostics', 'fitoptions','Ainit');
end

%Drawing pictures
if plotflag==true

figure(1);%distances of new individuals
clf; plot(aggdescriptives{2},'r'); hold on; plot(diagnostics{1}); axis([1 12 0 max(max(aggdescriptives{2}),max(diagnostics{1}))]);plot(12,aggdescriptives{2}(end),'r*'); plot(12,diagnostics{1}(end),'k*');
title('Distances of new individuals');
if rmflag==true
plot(diagrmodel{1},'m');
end

%degrees of new individuals
figure(2); 
clf; plot(aggdescriptives{4},'r'); hold on; plot(diagnostics{2}); axis([1 7 0 1]);
title('Degrees of new individuals');
if rmflag==true
plot(diagrmodel{2},'m');
end

%degree distribution of people who have lost positions
if alphaflag==true
figure(3); 
clf; plot(aggdescriptives{10},'r'); hold on; plot(p0.*exp(-alpha.*([1:1:length(aggdescriptives{10})]-1))); axis([1 length(aggdescriptives{10}) 0 max(aggdescriptives{10})]);
title('Degrees of old individuals');
end
end
