% NEWRANDPEOPLE generates new people to open vacancies.
% INPUTS:
% A=nitial adjacency matrix of a two-mode network,
% firms is a list of firms in which there are open vacancies,
% idvs is a cell array containing the indices of persons who are to be replaced
% in each firm, it is assumed that there is only one person per firm,
% nolist (optional) is a list of people who cannot be recruited,
% allpeople is the list of all people in the neighborhoods of a firm 
% (these can be computed by findallneighbors),
% dist and degr are their distances and degrees,
% outpeople contains the indices of people not in the neighborhoods,
% degroutpeople contains the degrees of outpeople,
% betap, gammap, pn, K, dinf are the model parameters
% rstate is the state of the random number generator
% OUTPUTS: 
% index contains indices of new board members for those who are found in 
% the network
% dstnew contains the corresponding distance from the vacancy
% dgrnew is the vector of degrees (number of positions) of newly recruited people, 
% if a person gets multiple appointments her/his degree is counted several times
% Anew is the updated adjacency matrix
% distance is set to m (number of firms) if a person is found in the network, 
% but not in any of the neighborhoods,
% if the person is completely new, the degree and distance are set to zero
% rstate is the state of the random number generator
% REQUIREMENTS: fastsetdiff
function [index,dstnew,dgrnew,Anew,rstate]=newrandpeople(A,firms,idvs,nolist,allpeople,dist,degr,outpeople,degroutpeople,betap,gammap,pn,K,dinf,rstate)

%Initializations
if nargin>=15 && ~isempty(rstate)
rand('state',rstate);
end

n=size(A,2);
m=size(A,1);
nof=length(firms);
index=zeros(nof,1);
dstnew=index;%Note: distance is set to zero initially, hence a completely new person gets distance zero
dgrnew=index;
Anew=A;

%Loop over firms with vacancies
for k=1:nof
indivs=idvs{k};
disttemp=dist{k};
degrtemp=degr{k};
outpeopletemp=outpeople{k};
degroutpeopletemp=degroutpeople{k};
allpeopletemp=allpeople{k};

if ~isempty(nolist) %nolist is people that should not be used in recruiting
[tempoutlist,indx]=fastsetdiff(allpeople{k},nolist);
allpeopletemp=allpeopletemp(indx);%This is the set of people who are in the neighborhoods of a company
disttemp=disttemp(indx);%This vector contains the distances
degrtemp=degrtemp(indx);%..and this one their degrees

[tempoutlist,indx]=fastsetdiff(outpeople{k},nolist);%people who are not in any neighborhood of the company
outpeopletemp=outpeopletemp(indx);
degroutpeopletemp=degroutpeopletemp(indx);%degrees of those people
end

if rand>=pn && ~(isempty(outpeopletemp) & isempty(disttemp))
%if rand>=pn then take the new person from the network
%the latter test is to check that there are people who can be recruited (for small networks this test may really matter)

temp=[];
if ~isempty(outpeopletemp) && ~isempty(disttemp)
temp2=exp(-gammap.*disttemp+betap.*min(degrtemp,K)./K);
temp=[temp2 exp(-gammap.*dinf+betap.*min(degroutpeopletemp,K)./K)];
elseif ~isempty(disttemp) && isempty(outpeopletemp)
temp=exp(-gammap.*disttemp+betap.*min(degrtemp,K)./K);
elseif isempty(disttemp) && ~isempty(outpeopletemp)
temp=[exp(-gammap.*dinf+betap.*min(degroutpeopletemp,K)./K)];
end%end of if ~isempty(outpeopletemp) && ~isempty(dist{k})

%Generating the index of the new person
thresholds=cumsum(temp);
randnum=rand*thresholds(end);
ind=min(find(randnum<=thresholds));

if ind<=length(allpeopletemp) %in this case the person is someone found in the neighborhoods
index(k)=allpeopletemp(ind);
dstnew(k)=disttemp(ind);
dgrnew(k)=degrtemp(ind);
Anew(firms(k),indivs)=0;
Anew(firms(k),index(k))=1;
else

index(k)=outpeopletemp(ind-length(allpeopletemp));
dstnew(k)=m; %note for an outsider (people in the network but not in any neighborhood) the dstnew is set to its max=m
dgrnew(k)=degroutpeopletemp(ind-length(allpeopletemp));
Anew(firms(k),indivs)=0;
Anew(firms(k),index(k))=1;
end %if for ind<=length(allpeopletemp)

else %rand<=pn in which case an outsider is chosen, these added in the end of Anew
Anew(firms(k),indivs)=0;
Anew(firms(k),end+1)=1;
%note in that case degr and dist are zeros
end %end of if rand>=pn
end %end of loop over firms with vacancies

rstate=rand('state');

