% SIMULATOR Function for simulating the exponential search model model. 
% The simulation runs at most ntime periods and at least msim periods. 
% The simulation may become terminated prior to running ntime periods 
% if all the relevant output distributions (degrees of all people, degrees
% and distances of newly recruited people, and degrees of people who have 
% left their positions) have converged. If you want to run N periods 
% but ignore the first M periods in the computation of the output 
% distributions set ntime=N and inittransient=M. Note that the initial 
% period is counted as one of the periods. The simulation may become 
% terminated prior to reaching ntime when the output distributions have 
% converged. If you want to force the simulation to run at least a given 
% number of periods set msim positive value. For example ntime=20 and 
% msim=5 will produce simulations that run at least 5 periods.
% INPUTS:
% Ainit is the initial adjacency matrix
% the parameters are as in the model, for explanation see the help of 
% fitsearchmodel.m,
% ntime is the number of periods / simulation,
% yr is the first year (can be left empty when Ainit is given, only needed
% to create Ainit when Ainit is empty),
% inittransient is the length of the initial transient in the simulation, 
% i.e., periods before inittransient are not included in the output 
% distribution, i.e., no initial transient means inittransient=1 (default),
% msim is the least number of periods / simulation,
% nn is the number of simulation runs, the outputs are averaged over the runs 
%(default=1)
% rstate is the state of the random number generator (see the help of rand)
% OUTPUTS:
% A adjacency matrix at the end of the simulations 
% Afout is the adjacency matrix of the initial firm network with weights 
% corresponding to number of times in which the firms are connected to each 
% other through board interlockers,
% diagnostics is a cell array containing the following fields
% degdis is the degree distribution of all people (degree=number of positions)
% avrccoef = average clustering coefficient over the simulation periods for 
% the firm projection,
% diagnostics{1} histogram of distances of newly recruited people 
% diagnostics{2} histogram of degrees of newly recruited people 
% diagnostics{3} histogram of degrees of people who have left their position
% diagnostics{4} simulated pn
% diagnostics{5} simulated p0
% rundata is a cell array containig some descriptives over the simulation runs,
% i.e., data of distributions of each period in the simulation;
% rundata{1}=degree distribution of all people
% rundata{2}=distances of newly recruited people
% rundata{3}=degrees of newly recruited people
% rundata{4}=degrees of people who have left their positions
% REQUIREMENTS: add2dist, averagehist, clustCoeffAlt, fastsetdiff, 
% findallneighbors, newrandpeople, openvacancies terminationtest, updateadj
function [A,Afout,degdis,avrccoef,diagnostics,rundata]=simulator(Ainit,pn,pout,betap,gammap,dinfty,K,ntime,yr,inittransient,msim,nn,cmax,rstate)

if nargin<14 || isempty(rstate) %just in case there is need to initialize the random number generator
rstate=rand('state'); 
end

if nargin<12 || isempty(nn)
nn=1;
end

if nargin<13 || isempty(cmax)
cmax=[];
end

if isempty(Ainit)
Ainit=initmatgc(yr);
end

if isempty(betap)
betap=0;
end

if isempty(gammap)
gammap=0;
end

if isempty(K)
K=1;

end

if isempty(dinfty)
dinfty=1;
end

if nargin<10 || isempty(inittransient) || inittransient<1
inittransient=1;%this means no initial transient
end

if nargin<11 || isempty(msim)
msim=0;
end

%initializations
Afout=spones(Ainit*Ainit');
stopflag=false;
minsim=max([msim inittransient]); %after the initial transient, this number of periods should pass, at least the initial transient is always run.
%the simulation runs no more than ntime periods
minsim=min(ntime,minsim);

%MAIN LOOP
%nn is the number of simulations to be run
for j=1:nn
distds{j}=[];
degnewds{j}=[];
degoldds{j}=[];
degdiss{j}=[];
A=Ainit;
dis0prev=[];
dis1prev=[];
dis2prev=[];
dis3prev=[];
clusvec=[];
totaldegr=[];
stopflag=false;
cntr=1;

%Simulation over one spell
while stopflag==false && cntr<=ntime
[A,distnew,degnew,degold,rstate]=updateadj(A,betap,gammap,pout,pn,K,dinfty,cmax,rstate);
Afout=Afout+spones(A*A');

if cntr>=inittransient %if initial transient has passed, update output distributions

clusvec=[clusvec clustCoeffAlt(spones(A*A')-diag(ones(size(A,1),1)))];
ndegr=sum(A,1);
if ~isempty(totaldegr)
totaldegr=add2dist(ndegr,totaldegr,0);
else
totaldegr=histc(ndegr,[1:1:max(ndegr)]);
end

distds{j}=add2dist(distnew,distds{j},1);%distance data
%remember distnew gives value m for those not found in the neighborhoods
degnewds{j}=add2dist(degnew,degnewds{j},1); 
degoldds{j}=add2dist(degold,degoldds{j},1);

dis0new=totaldegr./(cntr-inittransient+1);
degdiss{j}=dis0new; %degree distribution of all people

%Aggregated data; distances and degrees of new people and the degrees of people who left their positions.
if sum(distds{j})>0
dis1new=distds{j}./sum(distds{j});
else
dis1new=[];
end

if sum(degnewds{j})>0
dis2new=degnewds{j}./sum(degnewds{j}); %value zero when not found or an outsider
else
dis2new=[];
end

if sum(degoldds{j})>0
dis3new=degoldds{j}./sum(degoldds{j});
else
dis3new=[];
end

%Termination test, i.e., test whether the distributions have converged or not
if sum(distds{j})>0 && sum(degnewds{j})>0 && sum(degoldds{j})>0
testval=max([terminationtest(dis0new,dis0prev) terminationtest(dis1new,dis1prev) terminationtest(dis2new,dis2prev) terminationtest(dis3new,dis3prev)]);
else
testval=Inf;
end

if testval<eps && cntr>=minsim
stopflag=true;
else
dis0prev=dis0new;
dis1prev=dis1new;
dis2prev=dis2new;
dis3prev=dis3new;
end
end%end of main if (inittransient)

cntr=cntr+1;
end%end of main while loop

avrccoefs(j)=mean(clusvec);
end%end of for loop

%Gathering outputs
avrccoef=mean(avrccoefs);
Afout=Afout-diag(diag(Afout)); %substract the diagonal

rundata{1}=degdiss;
rundata{2}=distds;
rundata{3}=degnewds;
rundata{4}=degoldds;

[degdis,distd,degnewd,degoldd]=averagehist(degdiss,distds,degnewds,degoldds);

diagnostics{1}=distd(2:end)./sum(distd(2:end));%distd; remember the first element zero meaning not found
diagnostics{2}=degnewd(2:end)./sum(degnewd(2:end));
diagnostics{3}=degoldd;
diagnostics{4}=distd(1)/sum(distd); %simulated pn
diagnostics{5}=full(sum(degnewd)/((cntr-inittransient+1)*sum(sum(A))));%simulated p0
